# -*- project-name: VASM -*-
package VASM::Resource::Action;

use strict;
use warnings;
use base qw/Exporter/;
use VASM::Action;
use VASM::Resource;
use VASM::Resource::Message;
use IO::Handle;
use URI;
use File::MimeInfo::Magic;

our @EXPORT = qw/findActionCatalog
                 findActionCatalogByMedium
                 mediaDirectory/;
our $VERSION = '1.05';

sub findActionCatalog {
  # The resource path in which to find the action catalog
  my @path = @_;

  # Find the matching file with the highest precedence
  my $file = (findConfigResource('Action', @path, 'Action.xml'))[0];

  return unless $file; # Return nothing unless a file was found, otherwise:
  return VASM::Action->new($file); # Digest file and return the new catalog
}

sub findActionCatalogByMedium {
  # A URI or absolute local filename. Note that 'file:///home/hanuman' is
  # different from '/home/hanuman', because many programs still refuse to
  # accept the URI form (such as qiv, for instance).
  my ($path) = @_; my ($scheme, $mime, $catalog);

  # If $path is a valid URI, good, try to find an actions catalog for it:
  if (defined ($scheme = URI->new($path)->scheme())) {
    $catalog = findActionCatalog('URI', $scheme);
  } else { # By elimination, assume that this is a file
    # The file must exist for magic detection to work, obviously
    return unless defined ($mime = mimetype($path));
    $catalog = findActionCatalog('File', (split m!/!, $mime));

    # If that fails, check each superclass of the mimetype
    unless (defined $catalog) {
      for my $superclass (File::MimeInfo::mimetype_isa($mime)) {
        $catalog = findActionCatalog('File', (split m!/!, $superclass));
        # Exit if satisfied
        last if defined $catalog;
      }
    }
  }

  return $catalog; # Will be undef if nothing was found
}

# Return a VASM::Message object describing all media types usable by
# VASM::Action, both for MIME and URI data:
sub mediaDirectory {
  my $catalog = findMessageCatalog('Action', 'Directory', $_[0]);

  # The benefits of encapsulation
  return $catalog;
}

# Snippet left here for possible future use:
#   fileno(makeConfigResource('vasm', @{ $args{Path} }, 'Action.xml')), 'w');

1;

__END__
